"use strict";
// there are TWO classes here:
//  MeshHandle - represents a single BabyMesh, and usually a single BabylonMesh
//  MeshEngine - contains the collection of MeshHandles
Object.defineProperty(exports, "__esModule", { value: true });
exports.MeshHandle = exports.BabyEngine = void 0;
// every babyMesh points to a single meshHandle, which contains the
// information about that mesh.   the game writer gets to manipulate
// 'babyMesh' pointers, like "const myBox = app.shape('cube')", but can
// also create persistant shapes for which he does not have a pointer.
// there may be several BabylonJS objects attached to a single MeshHandle
const babylonjs_1 = require("babylonjs");
const tween_1 = require("./tween");
const babylon_1 = require("../backend/babylon");
const remote_1 = require("./remote");
const consoleRed = 'background-color:red;color:white;';
const consoleYellow = 'background-color:yellow;color:black;';
const consoleBlue = 'background-color:blue;color:white;';
class BabyEngine {
    constructor() {
        BabyEngine.shapeMap = new Map();
        BabyEngine.observers = {};
        BabyEngine.impostors = [];
        BabyEngine.babylonShapes = new babylon_1.Babylon(this); // can find a specific Babylon mesh by uuid too
        BabyEngine.frozen = false; // default.  freezes all tweens.
        // see if we are playerN or a spectator...
        this.url = new URL(window.location.href);
        console.log('URL', this.url);
        console.log('URL Search', this.url.search);
        let search = this.url.search.toLowerCase().trim(); // trim and lowercase
        switch (search) {
            // default (empty) is player0
            case '?s': // server only  
                BabyEngine.remote = new remote_1.Remote('server'); // also gets player0
                break;
            case '?p': // player 1-N
            case '?w': // watch / spectator   
                BabyEngine.remote = new remote_1.Remote('playerN');
                break;
            default:
                if (search !== '') {
                    console.error(`Did not expect URL + '${search}', valid values are <empty>, ?s (server) ?p (player), ?w (watcher`);
                }
                else { // standalone game
                    BabyEngine.remote = new remote_1.Remote('singlePlayer');
                }
        }
        // // come back to Ably later
        // if (true) {
        //     let ablyChannel = 'quickstart'
        //     BabyEngine.connectAbly(BabyEngine.ablyKey)
        //     BabyEngine.subscribeAbly(ablyChannel)
        //     BabyEngine.publishAbly(ablyChannel, 'hello')
        // }
        // keyCode is depreciated.  but it will be a long time before it
        // disappears.
        // document.addEventListener('keydown', event => {
        //     // console.log('key event fires', event)
        //     const data = { // call user's keyboard function
        //         keycode: event.keyCode,
        //         code: event.code,
        //         shiftKey: event.shiftKey,
        //         altKey: event.altKey,
        //         ctrlKey: event.ctrlKey,
        //         type: event.type,
        //         repeat: event.repeat,
        //         player: 0 // new !!
        //     }
        //     BabyEngine.notifyObservers('keydown', data)
        // })
        // document.addEventListener('keypress', event => {
        //     const data = { // call user's keyboard function
        //         code: event.code,
        //         shiftKey: event.shiftKey,
        //         altKey: event.altKey,
        //         ctrlKey: event.ctrlKey,
        //         type: event.type,
        //         repeat: event.repeat,
        //         player: 0 // new !!
        //     }
        //     BabyEngine.notifyObservers('keypress', data)
        // })
        // document.addEventListener('keyup', event => {
        //     const data = { // call user's keyboard function
        //         code: event.code,
        //         shiftKey: event.shiftKey,
        //         altKey: event.altKey,
        //         ctrlKey: event.ctrlKey,
        //         type: event.type,
        //         repeat: event.repeat,
        //         player: 0 // new !!
        //     }
        //     BabyEngine.notifyObservers('keyup', data)
        // })
    }
    // the 'gameroom' channel is used to set up the game
    // all players and spectators subscribe to 'updates', except p0 processes locally
    // p0 subscribes to 'keystrokesP1' 'mouseclicksP1, ,p2,p3 for keystrokes and mouseclicks
    // Shape commands sent to all players
    static broadcastToPlayers(cmd) {
        // console.log('broadcast ', cmd)
        cmd.timestamp = Date.now();
        if (cmd.duration < 0)
            cmd.duration = 0;
        // first, send it to any WebRTC clients
        // TODO:
        // then send it to the local babylonMesh class, which drives BabylonJS
        return this.processGameCmd(cmd);
    }
    static processGameCmd(cmd) {
        // console.log(`processGameCmd('${cmd.action}')`, cmd)
        // some commands, like 'inspector', can be handled immediately and safely return null
        switch (cmd.action) {
            case 'inspector':
                this.babylonShapes.inspector(cmd);
                return null; // but nothing else to do here
                break;
        }
        // everything we do from now on requires a mesh object. let's find it
        const handle = BabyEngine.shapeMap.get(cmd.uuid);
        console.assert(handle !== undefined, "Handle should not be undefined in ProcessGameCmd()" + JSON.stringify(cmd));
        // some commands, like 'createMesh' or 'setPosition' or 'getPosition 
        // can be handled immediately and do not need a tween
        switch (cmd.action) {
            case 'setCamera': // no tween necessary
            case 'setOrthographic':
            case 'setActive':
            case 'attachControls':
            case 'createBabylonMesh': // no tween necessary
            case 'button':
            case 'createGUI':
            case 'colorGUI':
            case 'positionGUI':
            case 'resizeGUI':
            case 'textGUI':
            case 'onPressedGUI':
            case 'drawGrid': // some commands for 2D drawing on a GRAPH box
            case 'drawLine':
            case 'drawCircle':
            case 'drawRect':
            case 'textAt':
            case 'setTexture':
            case 'dispose': // no tween necessary
            case 'addAxes':
            case 'addSkeleton': // currently unused
            case 'attachTo':
            case 'break':
            case 'collide':
            case 'penUp':
            case 'penDown':
            case 'lineErase':
            case 'setPosition':
            case 'addToPosition':
            case 'setRotation':
            case 'addToRotation':
            case 'print':
            case 'gameDispose':
                // console.log('babyengine', cmd)
                this.babylonShapes[cmd.action](cmd, handle); // eg: this.dispose(cmd)
                return null; // quick return
                break;
        }
        // ok, we have a handle (although it might be empty and might not be in the shapemap yet).
        // let's process it.
        let tween;
        const description = JSON.stringify(cmd);
        //////////////// special cases /////////////
        if (cmd.action == 'color' && !handle.hasDiffuseColor()) {
            console.error(`Cannot COLOR() this material`, description);
            return null;
        }
        /* eslint-disable no-fallthrough */
        switch (cmd.action) {
            case 'importShape':
                // TODO: add a promise for this
                tween = new tween_1.NullTween(description);
                this.babylonShapes.importShape(cmd, handle);
                break;
            case 'animatedGif':
            case 'move': // these use a V3Tween
            case 'moveToZero':
            case 'moveAndTurn':
            case 'moveToward':
            case 'turn':
            case 'moveToInitialPosition':
            case 'turnToZero':
            case 'turnToSame':
            case 'size':
            case 'color':
                // console.log('dispatching ', cmd.action)
                handle.description = description; // for debugging
                tween = new tween_1.V3Tween(JSON.stringify(cmd));
                // create a promise and store it away to be resolved someday
                tween.promise = new Promise(() => {
                    this.babylonShapes[cmd.action](cmd, tween, handle);
                });
                handle.tweenArray.push(tween);
                break;
            case 'opacity': // these use a scaler tween
                tween = new tween_1.ScalarTween(JSON.stringify(cmd));
                tween.promise = new Promise(() => {
                    this.babylonShapes[cmd.action](cmd, tween, handle);
                });
                handle.tweenArray.push(tween);
                break;
            default:
                console.error(`Did not expect '${cmd.action}`);
        }
        /* eslint-enable no-fallthrough */
        return tween;
    }
    // our observer class is adapted from the elegant Paon library
    //
    //  @author yahiko <yahiko.ninja@gmail.com>
    //  @link https://github.com/yahiko00/Paon
    static addObserver(type, observer, t = {}) {
        if (!(type in this.observers)) {
            this.observers[type] = [];
        }
        this.observers[type].push(observer.bind(t));
    }
    static notifyObservers(type, msg) {
        if (type in this.observers) {
            for (const obs of this.observers[type]) {
                obs(msg);
            }
        }
    }
    static removeObserver(type, observer) {
        // TODO: test this !!
        if (this.observers[type]) {
            for (let i = 0; i < this.observers[type].length; i++) {
                if (observer === this.observers[type][i]) {
                    this.observers[type].splice(i, 1);
                    return;
                }
            }
        }
    }
    static processTweens() {
        if (BabyEngine.frozen) // if the sceen is 'frozen', everything keeps running
            return; // except the tweens
        // for each entity in shapeMap    
        BabyEngine.shapeMap.forEach((handle) => {
            // run all the tweens for this entity
            handle.tweenArray.forEach((tween) => {
                tween.doTween();
            });
            // delete any expired tweens (remember to resolve promise)
            handle.tweenArray.forEach((tween, index) => {
                if (tween.expired()) {
                    // has to work for meshs, cameras, and lights, but only shapes has scaling
                    if (handle.babylonMesh instanceof babylonjs_1.Mesh) {
                        // different things for different types of tweens
                        if (tween.tweenType == 'size') {
                            // can't resize the physics impostor.  delete and recreate
                            // console.log('%cRECREATING PHYSICS IMPOSTOR',consoleYellow)
                            if (handle.babylonMesh.physicsImpostor !== null) {
                                this.babylonShapes.recreatePhysicsImpostor(handle);
                            }
                        }
                    }
                    // might not be a promise, if no 'await' / done()
                    console.assert(typeof tween.promiseResolve === 'function');
                    // console.log("%cResolving a promise", consoleRed)
                    tween.promiseResolve(); // complete the promise
                    handle.tweenArray.splice(index, 1); // and delete the tween
                }
            });
        });
    }
    static async getEnv() {
        await fetch('http://localhost:8080/dev.env')
            .then((response) => {
            if (response.status !== 200) {
                console.log('Looks like there was a problem. Status Code: ' +
                    response.status);
                return;
            }
            // Examine the text in the response
            response.json().then(function (data) {
                BabyEngine.ENV_FILE = new String(data);
                console.log('response', data);
            });
        })
            .catch((err) => {
            console.log('Fetch Error :-S', err);
        });
    }
}
exports.BabyEngine = BabyEngine;
// not sure when I should use an interface or a class.   I'm guessing a class
class MeshHandle {
    constructor(model, uuid) {
        const bottom = null;
        const top = null;
        this.uuid = uuid;
        this.model = model;
        this.thing = undefined;
        this.babylonMesh = null; // babylonMesh
        this.babylonRectangle = null;
        this.babyShape = null;
        this.material = null;
        this.dynamicTexture = null;
        this.bottom = bottom;
        this.top = top;
        this.material = null;
        this.size = new babylonjs_1.Vector3(1, 1, 1);
        this.diffuseColor = new babylonjs_1.Color3;
        this.opacity = 1;
        this.tweenArray = [];
        this.collider = 'solid';
        this.lastCollisionTime = 0;
        this.mass = 0;
        this.restitution = 0;
        this.uniqueName = model + '_' + uuid.toString();
        this.description = '';
        // for GUI elements
        this.fontsize = 18;
        // for lines 
        this.isPenDown = false;
        this.lastPosition = null;
        this.lineArray = [];
        this.radius = .2;
    }
    cleanBeforeDispose() {
        // a few cleanups, because Dispose doesn't know everything
        this.material.dispose();
        this.dynamicTexture.dispose();
        // if there is a line, dispose of each segment
        this.lineArray.forEach(element => {
            element.dispose();
        });
    }
    maybeAddLineSegment() {
        if (this.isPenDown) {
            if (this.lastPosition !== null) {
                // we need to add a line segment
                let path = [this.lastPosition, this.babylonMesh.position];
                let options = { path: path, radius: this.radius };
                let tubeSegment = babylonjs_1.MeshBuilder.CreateTube('', options);
                const material = new babylonjs_1.StandardMaterial('', tubeSegment._scene); // just use the tubebuilder's scene
                material.diffuseColor = this.diffuseColor;
                material.alpha = this.opacity;
                tubeSegment.material = material;
                this.lineArray.push(tubeSegment);
            }
            // for sure, we will create a tube next time
            this.lastPosition = this.babylonMesh.position.clone();
        }
    }
    getPosition() {
        return this.babylonMesh.position;
    }
    /** not all materials have a diffuseColor (eg: GridMaterial does not) */
    hasDiffuseColor() {
        let hasDiffuse = false;
        // has to work for meshs, cameras, and lights, but only shapes has scaling
        if (this.babylonMesh instanceof babylonjs_1.Mesh) {
            console.assert(this.babylonMesh.material !== null, 'no material ???'); // just checking....
            hasDiffuse = (typeof this.babylonMesh.material['diffuseColor'] !== 'undefined');
        }
        return (hasDiffuse);
    }
}
exports.MeshHandle = MeshHandle;
//# sourceMappingURL=babyengine.js.map